@@ -29,6 +29,7 @@ __copyright__ = "Copyright (c) 2004-2009 Sun Microsystems Inc." \
                 "Copyright (c) 2012 Igalia, S.L."
 __license__   = "LGPL"
 
+import importlib
 import os
 import pyatspi
 import re
@@ -59,6 +60,7 @@ except:
 from . import braille
 from . import debug
 from . import event_manager
+from . import messages
 from . import notification_messages
 from . import orca_state
 from . import script_manager
@@ -67,8 +69,6 @@ from . import settings_manager
 from . import speech
 from .input_event import BrailleEvent
 from .input_event import KeyboardEvent
-from .orca_i18n import _
-from .orca_i18n import ngettext
 
 _eventManager = event_manager.getManager()
 _scriptManager = script_manager.getManager()
@@ -240,10 +240,15 @@ def _processKeyboardEvent(event):
     # Special modes.
     if not isPressedEvent and keyboardEvent.event_string == "Escape":
         script.exitLearnMode(keyboardEvent)
+    if orca_state.learnModeEnabled and not keyboardEvent.modifiers:
+        if keyboardEvent.event_string == "F1":
+            orca_state.learnModeEnabled = False
+            return helpForOrca()
+        if keyboardEvent.event_string in ["1", "2"]:
+            orca_state.learnModeEnabled = False
+            return script.listOrcaShortcuts(keyboardEvent)
     if orca_state.capturingKeys:
         return False
-    if orca_state.listShortcutsModeEnabled:
-        return listShortcuts(keyboardEvent)
     if notification_messages.listNotificationMessagesModeEnabled:
         return notification_messages.listNotificationMessages(keyboardEvent)
 
@@ -425,12 +430,8 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
         try:
             speech.init()
             if reloaded and not skipReloadMessage:
-                # Translators: there is a keystroke to reload the user
-                # preferences.  This is a spoken prompt to let the user
-                # know when the preferences has been reloaded.
-                #
-                msg = _("Orca user settings reloaded.")
-                speech.speak(msg, settings.voices.get(settings.SYSTEM_VOICE))
+                speech.speak(messages.SETTINGS_RELOADED,
+                             settings.voices.get(settings.SYSTEM_VOICE))
             debug.println(debug.LEVEL_CONFIGURATION,
                           "Speech module has been initialized.")
         except:
@@ -467,8 +468,6 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
     _storeXmodmap(_orcaModifiers)
     _createOrcaXmodmap()
 
-    showMainWindowGUI()
-
     _scriptManager.activate()
     _eventManager.activate()
 
@@ -485,10 +484,7 @@ def showAppPreferencesGUI(script=None, inputEvent=None):
     """
 
     try:
-        module = __import__(settings.appGuiPreferencesModule,
-                            globals(),
-                            locals(),
-                            [''])
+        module = importlib.import_module('.app_gui_prefs', 'orca')
         module.showPreferencesUI()
     except:
         debug.printException(debug.LEVEL_SEVERE)
@@ -503,36 +499,13 @@ def showPreferencesGUI(script=None, inputEvent=None):
     """
 
     try:
-        module = __import__(settings.guiPreferencesModule,
-                            globals(),
-                            locals(),
-                            [''])
+        module = importlib.import_module('.orca_gui_prefs', 'orca')
         module.showPreferencesUI()
     except:
         debug.printException(debug.LEVEL_SEVERE)
 
     return True
 
-def showMainWindowGUI(script=None, inputEvent=None):
-    """Displays the Orca main window.
-
-    Returns True to indicate the input event has been consumed.
-    """
-
-    try:
-        module = __import__(settings.mainWindowModule,
-                            globals(),
-                            locals(),
-                            [''])
-        if settings.showMainWindow or settings.overrideDisabledMainWindow:
-            module.showMainUI()
-        else:
-            module.hideMainUI()
-    except:
-        debug.printException(debug.LEVEL_SEVERE)
-
-    return True
-
 def helpForOrca(script=None, inputEvent=None, page=""):
     """Show Orca Help window (part of the GNOME Access Guide).
 
@@ -546,174 +519,6 @@ def helpForOrca(script=None, inputEvent=None, page=""):
                  Gtk.get_current_event_time())
     return True
 
-def listShortcuts(event):
-    """When list shortcuts mode is enabled, this function provides a means
-    by which users can navigate through Orca bound Orca commands. Pressing
-    1 results in a list of the default shortcuts; pressing 2 results in a
-    list of shortcuts for the focused application, should one exist. List
-    navigation is accomplished through the Up and Down Arrow keys. Escape
-    exits the list. In this mode, other keys are disabled.
-
-    Arguments:
-    - event: an AT-SPI DeviceEvent
-
-    Returns True if the event is consumed (and False if not).
-    """
-
-    numShortcuts = len(orca_state.listOfShortcuts)
-    consumed = False
-    message = ""
-
-    # Translators: The following string instructs the user how to navigate
-    # amongst the list of commands presented in 'list shortcuts' mode as
-    # well as how to exit the list when finished.
-    #
-    navigation = \
-        _("Use Up and Down Arrow to navigate the list. Press Escape to exit.")
-
-    if event.type == pyatspi.KEY_PRESSED_EVENT:
-        if (event.event_string == "1"):
-            if not numShortcuts or orca_state.typeOfShortcuts != "default":
-                orca_state.listOfShortcuts = getListOfShortcuts("default")
-                orca_state.typeOfShortcuts = "default"
-                numShortcuts = len(orca_state.listOfShortcuts)
-            orca_state.ptrToShortcut = 0
-            # Translators: This message is presented when the user is in
-            # 'list of shortcuts mode'. In this messsage, we present the
-            # number of shortcuts found.
-            #
-            message = ngettext("%d Orca default shortcut found.",
-                               "%d Orca default shortcuts found.",
-                               numShortcuts) % numShortcuts
-            message = message + " " + navigation
-            orca_state.activeScript.presentMessage(message)
-            message = orca_state.listOfShortcuts[orca_state.ptrToShortcut][0]+ \
-              " " + orca_state.listOfShortcuts[orca_state.ptrToShortcut][1]
-            orca_state.activeScript.speakMessage(message)
-            orca_state.activeScript.displayBrailleMessage(message, -1, -1)
-            consumed = True
-        elif (event.event_string == "2"):
-            if not numShortcuts or orca_state.typeOfShortcuts != "application":
-                orca_state.listOfShortcuts = getListOfShortcuts("application")
-                orca_state.typeOfShortcuts = "application"
-                numShortcuts = len(orca_state.listOfShortcuts)
-            if numShortcuts > 0: 
-                orca_state.ptrToShortcut = 0
-                # Translators: This message is presented when the user is in
-                # 'list of shortcuts mode'. In this message, we present the
-                # number of shortcuts found for the named application.
-                #
-                message = ngettext( \
-                    "%(count)d Orca shortcut for %(application)s found.",
-                    "%(count)d Orca shortcuts for %(application)s found.",
-                    numShortcuts) % \
-                    {"count" : numShortcuts,
-                     "application" : orca_state.activeScript.app.name}
-                message = message + " " + navigation
-                orca_state.activeScript.presentMessage(message)
-                message = \
-                  orca_state.listOfShortcuts[orca_state.ptrToShortcut][0] + \
-                  " " + orca_state.listOfShortcuts[orca_state.ptrToShortcut][1]
-                orca_state.activeScript.speakMessage(message)
-                orca_state.activeScript.displayBrailleMessage(message, -1, -1)
-            else:
-                # Translators: This message is presented when the user is in
-                # 'list of shortcuts mode'. This is the message we present
-                # when the user requested a list of application-specific
-                # shortcuts, but none could be found for that application.
-                #
-                message = _("No Orca shortcuts for %s found.") % \
-                    (orca_state.activeScript.app.name)
-                orca_state.activeScript.speakMessage(message)
-                orca_state.activeScript.displayBrailleMessage(message, -1, -1)
-            consumed = True
-        elif (event.event_string == "Up"):
-            if (numShortcuts > 0):
-                if orca_state.ptrToShortcut > 0: 
-                    orca_state.ptrToShortcut = orca_state.ptrToShortcut-1
-                else:
-                    orca_state.ptrToShortcut = numShortcuts-1 
-                    # Translators: when the user is attempting to locate a
-                    # particular object and the top of a page or list is
-                    # reached without that object being found, we "wrap" to
-                    # the bottom and continue looking upwards. We need to
-                    # inform the user when this is taking place.
-                    #
-                    orca_state.activeScript.\
-                        presentMessage(_("Wrapping to bottom."))
-
-                message = \
-                  orca_state.listOfShortcuts[orca_state.ptrToShortcut][0] + \
-                  " " + orca_state.listOfShortcuts[orca_state.ptrToShortcut][1]
-                orca_state.activeScript.speakMessage(message)
-                orca_state.activeScript.displayBrailleMessage(message, -1, -1)
-            consumed = True
-        elif (event.event_string == "Down"):
-            if (numShortcuts > 0): 
-                if orca_state.ptrToShortcut < numShortcuts-1: 
-                    orca_state.ptrToShortcut = orca_state.ptrToShortcut+1
-                else:
-                    orca_state.ptrToShortcut = 0 
-                    # Translators: when the user is attempting to locate a
-                    # particular object and the bottom of a page or list is
-                    # reached without that object being found, we "wrap" to the
-                    # top and continue looking downwards. We need to inform the
-                    # user when this is taking place.
-                    #
-                    orca_state.activeScript.\
-                        presentMessage(_("Wrapping to top."))
-                message = \
-                  orca_state.listOfShortcuts[orca_state.ptrToShortcut][0] + \
-                  " " + orca_state.listOfShortcuts[orca_state.ptrToShortcut][1]
-                orca_state.activeScript.speakMessage(message)
-                orca_state.activeScript.displayBrailleMessage(message, -1, -1)
-            consumed = True
-        elif (event.event_string == "Escape"):
-            orca_state.activeScript.exitListShortcutsMode(event)
-            consumed = True 
-        else:
-            # Translators: Orca has a 'List Shortcuts' mode by which a user can
-            # navigate through a list of the bound commands in Orca. Pressing 1
-            # presents the commands/shortcuts available for all applications.
-            # These are the "default" commands/shortcuts. Pressing 2 presents
-            # commands/shortcuts Orca provides for the application with focus.
-            # The following message is presented to the user upon entering this
-            # mode.
-            #
-            message = _("Press 1 for Orca's default shortcuts. Press 2 for " \
-                        "Orca's shortcuts for the current application. " \
-                        "Press escape to exit.")
-            orca_state.activeScript.speakMessage(message)
-            orca_state.activeScript.displayBrailleMessage(message, -1, -1)
-            consumed = True
-    elif (event.type == pyatspi.KEY_RELEASED_EVENT) and (event.event_string \
-      == "Escape"):
-        consumed = True
-    return consumed
-
-
-def getListOfShortcuts(typeOfShortcuts):
-    """Returns a list of (description, shortcut) tuples, sorted by shortcut.
-
-    Arguments:
-    - typeOfShortcuts: a string specifying the desired type of shortcuts.
-      Valid values are 'default' for Orca-wide shortcuts and 'application'
-      for Orca shortcuts unique to the currently-focused application.
-    """
-
-    script = orca_state.activeScript
-    if typeOfShortcuts == "default":
-        bindings = script.getDefaultKeyBindings()
-        bound = bindings.getBoundBindings()
-    elif typeOfShortcuts == "application":
-        app = script.getAppKeyBindings()
-        toolkit = script.getToolkitKeyBindings()
-        bound = app.getBoundBindings()
-        bound.extend(toolkit.getBoundBindings())
-
-    shortcuts = [(kb.handler.description, kb.asString(True)) for kb in bound]
-    return sorted(shortcuts, key=lambda shortcut: shortcut[1])
-
 def quitOrca(script=None, inputEvent=None):
     """Quit Orca. Check if the user wants to confirm this action.
     If so, show the confirmation GUI otherwise just shutdown.
@@ -721,17 +526,7 @@ def quitOrca(script=None, inputEvent=None):
     Returns True to indicate the input event has been consumed.
     """
 
-    if _settingsManager.getSetting("quitOrcaNoConfirmation"):
-        shutdown()
-    else:
-        try:
-            module = __import__(settings.quitModule,
-                                globals(),
-                                locals(),
-                                [''])
-            module.showQuitUI()
-        except:
-            debug.printException(debug.LEVEL_SEVERE)
+    shutdown()
 
     return True
 
@@ -742,35 +537,11 @@ def showFindGUI(script=None, inputEvent=None):
     """
 
     try:
-        module = __import__(settings.findModule,
-                            globals(),
-                            locals(),
-                            [''])
+        module = importlib.import_module('.orca_gui_find', 'orca')
         module.showFindUI()
     except:
         debug.printException(debug.LEVEL_SEVERE)
 
-def showSplashGUI(script=None, inputEvent=None):
-    """Displays a splash screen.
-
-    Returns True to indicate the input event has been consumed.
-    """
-
-    try:
-        module = __import__(settings.splashModule,
-                            globals(),
-                            locals(),
-                            [''])
-        if _settingsManager.getSetting("showSplashWindow"):
-            module.showSplashUI()
-        else:
-            module.hideSplashUI()
-
-    except:
-        debug.printException(debug.LEVEL_SEVERE)
-
-    return True
-
 # If True, this module has been initialized.
 #
 _initialized = False
@@ -854,7 +625,7 @@ def timeout(signum=None, frame=None):
     debug.println(debug.LEVEL_SEVERE,
                   "TIMEOUT: something has hung.  Aborting.")
     debug.printStack(debug.LEVEL_ALL)
-    examineProcesses()
+    debug.examineProcesses()
     die(EXIT_CODE_HANG)
 
 def shutdown(script=None, inputEvent=None):
@@ -877,9 +648,7 @@ def shutdown(script=None, inputEvent=None):
         signal.signal(signal.SIGALRM, settings.timeoutCallback)
         signal.alarm(settings.timeoutTime)
 
-    # Translators: this is what Orca speaks and brailles when it quits.
-    #
-    orca_state.activeScript.presentMessage(_("Goodbye."))
+    orca_state.activeScript.presentMessage(messages.STOP_ORCA)
 
     _eventManager.deactivate()
     _scriptManager.deactivate()
@@ -954,34 +723,6 @@ def abortOnSignal(signum, frame):
                   % signum)
     die(signum)
 
-def getCmdline(pid):
-    try:
-        openFile = os.popen('cat /proc/%s/cmdline' % pid)
-        cmdline = openFile.read()
-        openFile.close()
-    except:
-        cmdline = '(Could not obtain cmdline)'
-    cmdline = cmdline.replace('\x00', ' ')
-
-    return cmdline
-
-def examineProcesses():
-    desktop = pyatspi.Registry.getDesktop(0)
-    debug.println(
-        debug.LEVEL_ALL, 'INFO: Desktop has %i apps:' % desktop.childCount)
-    for i, app in enumerate(desktop):
-        pid = app.get_process_id()
-        cmd = getCmdline(pid)
-        try:
-            name = app.name
-        except:
-            name = 'ERROR: Could not get name'
-        else:
-            if name == '':
-                name = 'WARNING: Possible hang'
-        debug.println(
-            debug.LEVEL_ALL, '%3i. %s (pid: %s) %s' % (i+1, name, pid, cmd))
-
 def main():
     """The main entry point for Orca.  The exit codes for Orca will
     loosely be based on signals, where the exit code will be the
@@ -1007,14 +748,15 @@ def main():
     init(pyatspi.Registry)
 
     try:
-        message = _("Welcome to Orca.")
+        message = messages.START_ORCA
         if not _settingsManager.getSetting('onlySpeakDisplayedText'):
             speech.speak(message, settings.voices.get(settings.SYSTEM_VOICE))
-        braille.displayMessage(message)
+        if _settingsManager.getSetting('enableBraille') \
+           or _settingsManager.getSetting('enableBrailleMonitor'):
+            braille.displayMessage(message)
     except:
         debug.printException(debug.LEVEL_SEVERE)
 
-    showSplashGUI()
     script = orca_state.activeScript
     if script:
         window = script.utilities.activeWindow()
